#include "requester.hpp"
#include <curl/curl.h>
#include <vector>
#include <cstring>
#include <stdexcept>
#include <utility>
#include <sstream>

using namespace sc;

size_t requester::write_data(char *buf, size_t sz, size_t nmemb, void *user_data) {
    size_t realsize = sz * nmemb;
    requester* req {reinterpret_cast<requester*>(user_data)};
    if (req->_logger) {
        SCInfo((*req->_logger), "received ", realsize, " bytes");
    }
    req->_text.append(buf, realsize);
    return realsize;
}

requester::requester() {
    curl_global_init(CURL_GLOBAL_ALL);
    _h.reset(curl_easy_init());
    curl_easy_setopt(_h.get(), CURLOPT_WRITEFUNCTION, write_data);
}

requester::~requester() {
    curl_global_cleanup();
}

requester::requester(requester&& other) :
    _h {std::move(other._h)}, _logger {other._logger}, _text {std::move(other._text)}
{
    other._h = nullptr;
    other._logger = nullptr;
    other._text = "";
}

requester& requester::operator=(requester&& other) {
    std::swap(_h, other._h);
    std::swap(_logger, other._logger);
    std::swap(_text, other._text);
    return *this;
}

std::string requester::get(const std::string &url) {
    _text = "";
    curl_easy_setopt(_h.get(), CURLOPT_URL, url.c_str());
    curl_easy_setopt(_h.get(), CURLOPT_WRITEDATA, this);
    auto success = curl_easy_perform(_h.get());
    if (success != CURLE_OK) {
        std::ostringstream oss;
        oss << "could not get remote data: " << curl_easy_strerror(success);
        throw std::runtime_error {oss.str()};
    }
    long status;
    success = curl_easy_getinfo(_h.get(), CURLINFO_RESPONSE_CODE, &status);
    if (success != CURLE_OK || status != 200) {
        std::ostringstream oss;
        oss << "could not get remote data (" << status << ")";
        throw std::runtime_error {oss.str()};
    }
    return _text;
}

std::string requester::post(const std::string& url, const std::string& json) {
    curl_easy_setopt(_h.get(), CURLOPT_URL, url.c_str());
    curl_easy_setopt(_h.get(), CURLOPT_POST, 1L);
    curl_easy_setopt(_h.get(), CURLOPT_POSTFIELDS, json.c_str());
    curl_easy_setopt(_h.get(), CURLOPT_POSTFIELDSIZE, json.size());

    struct curl_slist* headers {nullptr};
    headers = curl_slist_append(headers, "Content-Type: application/json");
    std::unique_ptr<struct curl_slist, void(*)(struct curl_slist*)> hdrs {headers, curl_slist_free_all};
    curl_easy_setopt(_h.get(), CURLOPT_HTTPHEADER, headers);

    std::string response;
    curl_easy_setopt(_h.get(), CURLOPT_WRITEDATA, &response);

    auto success = curl_easy_perform(_h.get());
    if (success != CURLE_OK) {
        std::ostringstream oss;
        oss << "could not post data: " << curl_easy_strerror(success);
        throw std::runtime_error {oss.str()};
    }
    return response;
}
